CodeMirror.defineMode('nginx-mini+regex', function(options) {
	var pcre_mode = CodeMirror.getMode(options, {name: 'pcre'});
	return {
		startState: function() {
			return {
				current: '',
				pcre_state: CodeMirror.startState(pcre_mode),
				expect_regex: false, // whether the next string should be a regex
			};
		},
		copyState: function(os) { // os = original state
			return {
				current: os.current,
				pcre_state: CodeMirror.copyState(pcre_mode, os.pcre_state),
				expect_regex: os.expect_regex,
			};
		},
		token: function(stream, state) {
			if (state.current === 'string' || state.current === 'regex') {
				if (stream.eat('"')) {
					state.current = '';
					return 'string';
				}
				if (state.current === 'regex') return pcre_mode.token(stream, state.pcre_state);
				if (!stream.match('\\"')) stream.next();
				return state.current;
			}
			var operator = stream.match(/(=|~|~\*|\^~)/);
			if (operator) {
				state.expect_regex = operator[1][0] === '~'; // ~ and ~* imply a regex
				return 'operator';
			}
			if (stream.eat('"')) {
				state.current = state.expect_regex ? 'regex' : 'string';
				state.expect_regex = false;
				return 'string';
			}
			/* Minimalistic nginx syntax highlighting, just for demonstration purposes: */
			if (stream.eat('#')) {
				stream.skipToEnd();
				return 'comment';
			}
			if (stream.match(/\b(?:if|return|server|location)\b/)) return 'keyword';
			if (stream.match(/\$\w+/)) return 'variable-3';
			if (stream.match(/\d+/)) return 'number';
			stream.next();
			return state.current;
		},
	};
});
